/* ------------------------------------------------------------------------------

S270418 - HW ref: A180118, A180118-R070318

Blinks the GPA0 led attached to the I2C expander



Notes:

The MCP23017 optional GPIO expander must be populated on the PCB

--------------------------------------------------------------------------------- */

// ------------------------------------------------------------------------------
//
//  Libraries
//
// ------------------------------------------------------------------------------

#include <Wire.h>

// ------------------------------------------------------------------------------
//
// Hardware definitions for the MCP23017 GPIO Expander
//
// ------------------------------------------------------------------------------

#define   GPIOEXP_ADDR  0x20  // MCP23017 I2C address
#define   IODIRA_REG    0x00  // MCP23017 internal register IODIRA  (see datasheet)
#define   IODIRB_REG    0x01  // MCP23017 internal register IODIRB  (see datasheet)
#define   GPPUA_REG     0x0C  // MCP23017 internal register GPPUA  (see datasheet)
#define   GPPUB_REG     0x0D  // MCP23017 internal register GPPUB  (see datasheet)
#define   GPIOA_REG     0x12  // MCP23017 internal register GPIOA  (see datasheet)
#define   GPIOB_REG     0x13  // MCP23017 internal register GPIOB  (see datasheet)
 
// ------------------------------------------------------------------------------

void setup()
{
  Wire.begin(0,2);
  Serial.begin(9600);
}
 
 
void loop()
{
  if (I2C_scanner() > 0)    // Check if any I2C device is present
  {
    Serial.println("Blinking GPIOA led...");
    while (1)
    {
      IODIRA_write(0xfe);   // Set GPA0 (MCP23017 pin 21) as output
      GPIOA_write(0x01);    // Set GPA0 (MCP23017 pin 21) HIGH
      delay(500);     
      GPIOA_write(0x00);    // Set GPA0 (MCP23017 pin 21) LOW
      delay(500);           
    }
  }
}

// ------------------------------------------------------------------------------

void GPIOA_write(byte ioData)
{
  Wire.beginTransmission(GPIOEXP_ADDR);
  Wire.write(GPIOA_REG);                // Select GPIOA
  Wire.write(ioData);                   // Write value
  Wire.endTransmission();
}

// ------------------------------------------------------------------------------

void IODIRA_write(byte ioData)
{
  Wire.beginTransmission(GPIOEXP_ADDR);
  Wire.write(IODIRA_REG);               // Select IODIRA
  Wire.write(ioData);                   // Write value
  Wire.endTransmission();
}

// ------------------------------------------------------------------------------

void GPPUA_write(byte ioData)
{
  Wire.beginTransmission(GPIOEXP_ADDR);
  Wire.write(GPPUA_REG);                // Select GPPUA
  Wire.write(ioData);                   // Write value
  Wire.endTransmission();
}

// ------------------------------------------------------------------------------

byte GPIOA_read()
{
  // Set MCP23017 pointer to GPIOA
  Wire.beginTransmission(GPIOEXP_ADDR);
  Wire.write(GPIOA_REG);
  Wire.endTransmission();
  // Read GPIOA
  Wire.beginTransmission(GPIOEXP_ADDR);
  Wire.requestFrom(GPIOEXP_ADDR, 1);
  return Wire.read();
}

// ------------------------------------------------------------------------------

int I2C_scanner()
{
  byte error, address;
  int nDevices;

  Serial.println("\nI2C Scanner");
  Serial.println("Scanning...");
 
  nDevices = 0;
  for(address = 1; address < 127; address++ ) 
  {
 
    Wire.beginTransmission(address);
    error = Wire.endTransmission();
 
    if (error == 0)
    {
      Serial.print("I2C device found at address 0x");
      if (address<16) 
        Serial.print("0");
      Serial.print(address,HEX);
      Serial.println("  !");
 
      nDevices++;
    }
    else if (error==4) 
    {
      Serial.print("Unknow error at address 0x");
      if (address<16) 
        Serial.print("0");
      Serial.println(address,HEX);
    }    
  }
  if (nDevices == 0)
    Serial.println("No I2C devices found\n");
  else
    Serial.println("done\n");
  return nDevices;
}

